unit gMouseGesture;

{
  Mouse Gesture Component for Delphi

  written by Wolfy(http://hp.vector.co.jp/authors/VA024591/)
  license BSD
  modified 2002/10/14

  thanks to OpenJane, this code had been written.
}



{$IFDEF VER130}
{$ELSE}
  {$WARN SYMBOL_PLATFORM OFF}
  {$WARN UNIT_PLATFORM OFF}
{$ENDIF}

{..$DEFINE IGNORE_RIGHT_CLICK}  //ListviewTreeView̃ZNgŗL


interface

uses
  Windows, Messages, SysUtils, Classes,controls,appevnts;

type
  TMouseGestureType = (mgUp,mgDown,mgLeft,mgRight,
                       mgLeftClick,mgWheelClick,mgWheelUp,mgWheelDown,
                       mgSide1,mgSide2,mgSide1R,mgSide2R);
  TMouseGestureArray = array of TMouseGestureType;    

  TMouseGestureExecuteEvent = procedure(Sender: TObject; Gestures: TMouseGestureArray) of object;
  TMouseGestureNotifyEvent = procedure(Sender: TObject; Gesture: TMouseGestureType) of object;

  TgMouseGesture = class(TCustomApplicationEvents)
  private
    FPrevPoint: TPoint;
    FClickGestureStandby: Boolean;
    FMargin: Integer;
    FTempMoveGestures: TMouseGestureArray;
    FEnabled: Boolean;

    FOnMouseGestureExecute: TMouseGestureExecuteEvent;
    FOnMouseGestureNotify: TMouseGestureNotifyEvent;
    FOnMouseGestureNotifyMove: TMouseGestureExecuteEvent;
  protected
    procedure DoMouseGesture(var Msg: TMsg; var Handled: Boolean); virtual;
    procedure MoveGesture(pt: TPoint);
    procedure GestureExecute(Gestures: TMouseGestureArray);
    function AddGesture(Gesture: TMouseGestureType; Ary: TMouseGestureArray = nil): TMouseGestureArray;
  public
    constructor Create(AOwner: TComponent); override;
  published
    //}EXړF鋗
    property Margin: Integer read FMargin write FMargin;
    //L/
    property Enabled: Boolean read FEnabled write FEnabled;

    property OnMouseGestureExecute: TMouseGestureExecuteEvent read FOnMouseGestureExecute write FOnMouseGestureExecute;
    property OnMouseGestureNotify: TMouseGestureNotifyEvent read FOnMouseGestureNotify write FOnMouseGestureNotify;
    property OnMouseGestureNotifyMove: TMouseGestureExecuteEvent read FOnMouseGestureNotifyMove write FOnMouseGestureNotifyMove;
  end;


function MouseGestureToString(Gesture: TMouseGestureType): String;
function MouseGestureArrayToString(Gestures: TMouseGestureArray): String;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Main1', [TgMouseGesture]);
end;

const
  CRLF = sLineBreak;
  WM_CONTROLRECREATED = WM_USER + 020103;
  WM_XBUTTONDOWN   = $020B;
  WM_XBUTTONUP     = $020C;
  WM_XBUTTONDBLCLK = $020D;
  MK_XBUTTON1 = $0001;
  MK_XBUTTON2 = $0002;

function MouseGestureToString(Gesture: TMouseGestureType): String;
begin
  case Gesture of
    mgUp: Result := '';
    mgDown: Result := '';
    mgLeft: Result := '';
    mgRight: Result := '';
    mgLeftClick: Result := 'LeftClick';
    mgWheelClick: Result := 'WheelClick';
    mgWheelUp: Result := 'WheelUp';
    mgWheelDown: Result := 'WheelDown';
    mgSide1: Result := 'Side1';
    mgSide2: Result := 'Side2';
    mgSide1R: Result := 'Side1R';
    mgSide2R: Result := 'Sede2R';
  end;
end;

function MouseGestureArrayToString(Gestures: TMouseGestureArray): String;
var
  i: Integer;
  g: TMouseGestureType;
begin
  for i := 0 to Length(Gestures) - 1 do
  begin
    g := Gestures[i];
    case g of
      mgUp,mgDown,mgLeft,mgRight:
        Result := Result + MouseGestureToString(g);
    else
      if Result = '' then
        Result := MouseGestureToString(g)
      else
        Result := Result + CRLF + MouseGestureToString(g);
    end;
  end;
end;
  

{ TgMouseGesture }

function TgMouseGesture.AddGesture(
  Gesture: TMouseGestureType; Ary: TMouseGestureArray): TMouseGestureArray;
//WFX`[
begin
  Result := Ary;
  if Length(Result) = 0 then
  begin
    //쐬
    SetLength(Result,1);
    Result[0] := Gesture;
  end
  else begin
    //ǉ
    if Result[High(Result)] <> Gesture then
    begin
      SetLength(Result,Length(Result) + 1);
      Result[High(Result)] := Gesture;
    end;
  end;
end;

constructor TgMouseGesture.Create(AOwner: TComponent);
begin
  inherited;
  FMargin := 5;
  FEnabled := True;
  //CxgZbg
  OnMessage := DoMouseGesture;
end;

procedure TgMouseGesture.DoMouseGesture(var Msg: TMsg;
  var Handled: Boolean);
//}EXWFX`[
begin
  if not FEnabled then
    Exit;

  case Msg.message of
    WM_RBUTTONDOWN: //Jn
    begin
      FTempMoveGestures := nil;
      FPrevPoint := Msg.pt;
{$IFDEF IGNORE_RIGHT_CLICK}
      Handled := True;
{$ELSE}
      Handled := False;
{$ENDIF}       
    end;

    WM_RBUTTONUP: //s
    begin
      if (Length(FTempMoveGestures) > 0) and (Msg.wParam = 0) then
      begin
        GestureExecute(FTempMoveGestures);
        Handled := True;
      end
      else
        Handled := False;

      FClickGestureStandby := False;
    end;

    WM_MOUSEMOVE: //WFX`[`FbN
      if Msg.wParam = MK_RBUTTON then
      begin
        MoveGesture(Msg.pt);
        Handled := True;
      end;

    WM_LBUTTONUP: //{^
      if FClickGestureStandby then
      begin
        FClickGestureStandby := False;
        Handled := True;
        GestureExecute(AddGesture(mgLeftClick));
      end;

    WM_MBUTTONUP: //zC[{^
      if FClickGestureStandby then
      begin
        FClickGestureStandby := False;
        Handled := True;
        GestureExecute(AddGesture(mgWheelClick));
      end;

    WM_XBUTTONUP: //{^
      if FClickGestureStandby then
      begin
        FClickGestureStandby := False;
        Handled := True;
        if (GetKeyState(VK_RBUTTON) < 0) then
        case HiWord(Msg.wParam) of
          MK_XBUTTON1: GestureExecute(AddGesture(mgSide1R));
          MK_XBUTTON2: GestureExecute(AddGesture(mgSide2R));
        end
        else begin
          case HiWord(Msg.wParam) of
            MK_XBUTTON1: GestureExecute(AddGesture(mgSide1));
            MK_XBUTTON2: GestureExecute(AddGesture(mgSide2));
          end;
        end;
      end;

    WM_LBUTTONDOWN:
      if Msg.wParam = (MK_RBUTTON + MK_LBUTTON) then
      begin
        FClickGestureStandby := True;
        Handled := True;
      end;

    WM_MBUTTONDOWN:
      if Msg.wParam = (MK_RBUTTON + MK_MBUTTON) then
      begin
        FClickGestureStandby := True;
        Handled := True;
      end;

    WM_XBUTTONDOWN:
      if (HiWord(Msg.wParam) = MK_XBUTTON1) or
         (HiWord(Msg.wParam) = MK_XBUTTON2) then
      begin
        FClickGestureStandby := True;
        Handled := (GetKeyState(VK_RBUTTON) < 0);
      end;

    WM_MOUSEWHEEL:
      if LoWord(Msg.wParam) = MK_RBUTTON then
      begin
        if (Msg.wParam > 0) then
          GestureExecute(AddGesture(mgWheelUp))
        else
          GestureExecute(AddGesture(mgWheelDown));

        Handled := True;
      end;
  end;

end;

procedure TgMouseGesture.GestureExecute(Gestures: TMouseGestureArray);
//s
begin
  if Assigned(FOnMouseGestureExecute) then
    FOnMouseGestureExecute(Self,Gestures);
end;

procedure TgMouseGesture.MoveGesture(pt: TPoint);
//WFX`[`FbN
var
  temp: Extended;
  distance: TPoint;
  arrow: TMouseGestureType;
  len: Integer;
begin
  distance.X := Abs(pt.X - FPrevPoint.X);
  distance.Y := Abs(pt.Y - FPrevPoint.Y);
  if (distance.X > FMargin) or
     (distance.Y > FMargin) then
  begin
    temp := distance.Y / (distance.X + 0.001);
    if temp >= 1 then
    begin
      if pt.Y > FPrevPoint.Y then
        arrow := mgDown
      else
        arrow := mgUp;
    end else
    begin
      if pt.X > FPrevPoint.X then
        arrow := mgRight
      else
        arrow := mgLeft;
    end;

    len := Length(FTempMoveGestures);
    FTempMoveGestures := AddGesture(arrow,FTempMoveGestures);
    if len <> Length(FTempMoveGestures) then
    begin
      //Cxg
      if Assigned(FOnMouseGestureNotify) then
        FOnMouseGestureNotify(Self,arrow);

      if Assigned(FOnMouseGestureNotifyMove) then
        FOnMouseGestureNotifyMove(Self,FTempMoveGestures);
    end;

    FPrevPoint := pt;
  end;
end;


end.
